function [p, q] = findNE(A, B)
%
% Find the Nash Equilibria (NE) for the two-players normal form game. The
% algorithm is based on finding the supports of NE.
%
%  -- INPUT PARAMETERS --
%  A: [MxN] payoffs for the first player
%  B: [MxN] payoffs for the second player 
%
%  -- OUTPUT PARAMETERS --
%  p: [MxL] NE for the first player (L is the number of NE)
%  q: [NxL] NE for the second player 
%
% (c) Yannis Pantazis, 2010
% mail: yannis.pantazis@gmail.com

[M, N] = size(A);
m = 1:M;
n = 1:N;

p = [];
q = [];
for k = 1:min(M, N)
    C1 = nchoosek(m, k);
    C2 = nchoosek(n, k);
    
    for i = 1:size(C1, 1)
        for j = 1:size(C2, 1)
            % construct the system of equations
            % first player
            s_p = zeros(k,1);
            s_p(1) = 1;
            
            S_p = zeros(k,k);
            S_p(1,:) = ones(1,k);
            S_p(2:k, :) = repmat(B(C1(i,:),C2(j,1))', k-1, 1);
            for l = 2:k
                S_p(l,:) = S_p(l,:) - B(C1(i,:),C2(j,l))';
            end
            
            % second player
            s_q = zeros(k,1);
            s_q(1) = 1;
            
            S_q = zeros(k,k);
            S_q(1,:) = ones(1,k);
            S_q(2:k, :) = repmat(A(C1(i,1),C2(j,:)), k-1, 1);
            for l = 2:k
                S_q(l,:) = S_q(l,:) - A(C1(i,l),C2(j,:));
            end

            
            % solve the system of eq.
            if rank(S_p)==k & rank(S_q)==k
                p_cand = zeros(M,1);
                p_cand(C1(i,:)) = S_p\s_p;

                q_cand = zeros(N,1);
                q_cand(C2(j,:)) = S_q\s_q;


                % check if the solution is a NE
                if p_cand >= 0 
                if q_cand >= 0
                    p_payoff = A(C1(i,1),:)*q_cand;
                    q_payoff = B(:,C2(j,1))'*p_cand;

                    p_br = A(setdiff(m, C1(i,:)),:)*q_cand;
                    q_br = B(:,setdiff(n, C2(j,:)))'*p_cand;


                    if (isempty(p_br) | p_payoff>=max(p_br)) & (isempty(q_br) | q_payoff>=max(q_br))
                        p = [p, p_cand];
                        q = [q, q_cand];
                    end
                end
                end
            end
        end
    end
end


